package org.jbpm.examples.evaluation;

import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.Operation;
import org.jbpm.task.service.TaskService;

import bitronix.tm.resource.jdbc.PoolingDataSource;


public class PersistentProcessExample 
{
	public static final void main(String[] args) {
		try {
			//Initialize task service
			Map<String, String> props1 = new HashMap<String, String>();
			props1.put("hibernate.dialect", "org.hibernate.dialect.MySQLDialect");
			props1.put("hibernate.connection.driver_class", "com.mysql.jdbc.Driver");
			props1.put("hibernate.connection.url", "jdbc:mysql://localhost:3306/jbpm");
			props1.put("hibernate.connection.username", "root");
			props1.put("hibernate.connection.password", "root");			
			EntityManagerFactory emf1 = Persistence.createEntityManagerFactory("org.jbpm.task", props1);
			TaskService taskService = new TaskService(emf1, SystemEventListenerFactory.getSystemEventListener());
			
			//Initialize JTA data source 
			Class.forName("org.gjt.mm.mysql.Driver");			
			PoolingDataSource ds = new PoolingDataSource();
			ds.setUniqueName("jdbc/processInstanceDS");
			ds.setClassName("com.mysql.jdbc.jdbc2.optional.MysqlXADataSource");	
			ds.setMaxPoolSize(3);
			ds.setAllowLocalTransactions(true);
			ds.getDriverProperties().put("user", "root");
			ds.getDriverProperties().put("password", "root");
			ds.getDriverProperties().put("URL", "jdbc:mysql://localhost:3306/jbpm");
			ds.init();

			//Initialize persistent process engine
			EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");			
			KnowledgeBase kbase = readKnowledgeBase();								
			Environment env = KnowledgeBaseFactory.newEnvironment();
			env.set( EnvironmentName.ENTITY_MANAGER_FACTORY, emf );
			// create a new knowledge session that uses JPA to store the runtime state
			StatefulKnowledgeSession processSession = JPAKnowledgeService.newStatefulKnowledgeSession( kbase, null, env );			
						
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(processSession, "test", 1000);		
			
			//Register process session to local task service session
			LocalHumanTaskHandler handler = new LocalHumanTaskHandler(taskService); 
			processSession.getWorkItemManager().registerWorkItemHandler("Human Task", handler);	
						
			Map<String, Object> params = new HashMap<String, Object>();
			params.put("employee", "krisv");
			params.put("reason", "Yearly performance evaluation");						
			//Start new process
			ProcessInstance process = processSession.startProcess("com.sample.evaluation", params);			
			
			
			TaskSummary task2 = handler.getTaskSession().getTasksAssignedAsPotentialOwner("krisv", "en-UK").get(0);			
			System.out.println("User executing task " + task2.getName() + "(" + task2.getId() + ": " + task2.getDescription() + ")");		
			handler.getTaskSession().taskOperation(Operation.Start, task2.getId(), "krisv", null, null, null);
			handler.getTaskSession().taskOperation(Operation.Complete, task2.getId(), "krisv", null, null, null);
					
			processSession.dispose();
			logger.close();
			
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {		
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("evaluation/Evaluation.bpmn"), ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}


}


